@@ -787,7 +787,7 @@ ebb_webdav_uid_to_uri (EBookBackendWebDAV *bbdav,
 {
 	ESourceWebdav *webdav_extension;
 	SoupURI *soup_uri;
-	gchar *uri, *tmp, *filename;
+	gchar *uri, *tmp, *filename, *uid_hash = NULL;
 
 	g_return_val_if_fail (E_IS_BOOK_BACKEND_WEBDAV (bbdav), NULL);
 	g_return_val_if_fail (uid != NULL, NULL);
@@ -796,6 +796,17 @@ ebb_webdav_uid_to_uri (EBookBackendWebDAV *bbdav,
 	soup_uri = e_source_webdav_dup_soup_uri (webdav_extension);
 	g_return_val_if_fail (soup_uri != NULL, NULL);
 
+	/* UIDs with forward slashes can cause trouble, because the destination server
+	   can consider them as a path delimiter. For example Google book backend uses
+	   URL as the contact UID. Double-encode the URL doesn't always work, thus
+	   rather cause a mismatch between stored UID and its href on the server. */
+	if (strchr (uid, '/')) {
+		uid_hash = g_compute_checksum_for_string (G_CHECKSUM_SHA1, uid, -1);
+
+		if (uid_hash)
+			uid = uid_hash;
+	}
+
 	if (extension) {
 		tmp = g_strconcat (uid, extension, NULL);
 		filename = soup_uri_encode (tmp, NULL);
@@ -822,6 +833,7 @@ ebb_webdav_uid_to_uri (EBookBackendWebDAV *bbdav,
 
 	soup_uri_free (soup_uri);
 	g_free (filename);
+	g_free (uid_hash);
 
 	return uri;
 }
